/*============================================================================
  File:     05_MoreInfo.sql.sql

  SQL Server Versions: 2008, 2008R2, 2012, 2014, 2016
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/
/*
	set up
	ALTER INDEX SD1 ON [Sales].[SalesOrderDetailB] DISABLE

	ALTER INDEX SD2 ON [Sales].[SalesOrderDetailB] DISABLE


	CREATE NONCLUSTERED INDEX [IX_SalesOrderDetailB_ProductID] ON [Sales].[SalesOrderDetailB]
	(
		[ProductID] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	GO

*/
USE [AdventureWorks2016];
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_GetProductInfo];
GO

CREATE PROCEDURE [Sales].[usp_GetProductInfo]
	@ProductID INT
AS	

	SELECT [ProductID], [OrderQty]
	FROM [Sales].[SalesOrderDetailB]
	WHERE [ProductID] = @ProductID
	ORDER BY [ProductID];

GO

/*
	try it with the user's value
*/
EXEC [Sales].[usp_GetProductInfo] 1870;


/*
	find the plan in cache
	check the input parameters
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text], 
		[plan].[query_plan]
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
CROSS APPLY [sys].[dm_exec_query_plan]([cp].[plan_handle]) [plan]
WHERE [dest].[text] LIKE '%GetProductInfo%'
OPTION(MAXDOP 1, RECOMPILE);


/*
	either drop the plan from cache, or use RECOMPILE	
*/
EXEC [Sales].[usp_GetProductInfo] 1870 WITH RECOMPILE;
GO

/*
	Warnings...
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
WHERE [OrderDate] BETWEEN '2012-01-01 00:00:00.000' AND '2013-12-31 23:59:59.997'
ORDER BY [OrderDate]
OPTION (RECOMPILE);


/*
	Check out row and page count
	31465	733
*/
SELECT 
	OBJECT_NAME([p].[object_id]) [TableName], 
	[si].[name] [IndexName], 
	[au].[type_desc] [Type], 
	[p].[rows] [RowCount], 
	[au].total_pages [PageCount]
FROM [sys].[partitions] [p]
JOIN [sys].[allocation_units] [au] ON [p].[partition_id] = [au].[container_id]
JOIN [sys].[indexes] [si] on [p].[object_id] = [si].object_id and [p].[index_id] = [si].[index_id]
WHERE [p].[object_id] = OBJECT_ID(N'Sales.SalesOrderHeader');


/*
	Trick the optimize a bit here and change row count
	**Don't try this at home!!**
*/
UPDATE STATISTICS  [Sales].[SalesOrderHeader]
	WITH ROWCOUNT = 100, PAGECOUNT = 733;
GO


/*
	re-run
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
WHERE [OrderDate] BETWEEN '2012-01-01 00:00:00.000' AND '2013-12-31 23:59:59.997'
ORDER BY [OrderDate]
OPTION (RECOMPILE);

/*
	How to track these?
	In the default trace...
	C:\Program Files\Microsoft SQL Server\MSSQL12.IE2\MSSQL\Log

	Can we get better information through XEvents?  Yes ;)
*/
CREATE EVENT SESSION [HashSortWarnings] 
	ON SERVER 
	ADD EVENT sqlserver.hash_warning(
		ACTION(
			sqlserver.sql_text,sqlserver.tsql_stack)
			),
	ADD EVENT sqlserver.sort_warning(
		ACTION(
			sqlserver.sql_text,sqlserver.tsql_stack)
			),
	ADD EVENT sqlserver.sql_batch_completed
ADD TARGET package0.event_file(
	SET filename=N'C:\temp\SortWarnings',max_file_size=(512)
	)
WITH (MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,
MAX_DISPATCH_LATENCY=30 SECONDS,MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,
TRACK_CAUSALITY=ON,STARTUP_STATE=ON)
GO

ALTER EVENT SESSION [HashSortWarnings]
	ON SERVER
	STATE=START;
GO

/*
	re-run again
	view the data
*/
DBCC FREEPROCCACHE;

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
WHERE [OrderDate] BETWEEN '2012-01-01 00:00:00.000' AND '2013-12-31 23:59:59.997'
ORDER BY [OrderDate];

/*
	note that the XE session isn't perfect - you can't rely on sql_text
	either use tsql_stack and remove sql_batch_completed, or,
	remove tsql_stack and sql_text, and keep sql_batch_completed

	clean up 
*/
ALTER EVENT SESSION [HashSortWarnings]
	ON SERVER
	STATE=STOP;
GO

DROP EVENT SESSION [HashSortWarnings]
	ON SERVER;
GO

UPDATE STATISTICS  [Sales].[SalesOrderHeader]
	WITH ROWCOUNT = 31465, PAGECOUNT = 733;
GO




/* Environment reset

USE [AdventureWorks2014];
GO
DROP FUNCTION Sales.SalePrice;
GO

EXEC sp_configure 'show advanced options', 1;
RECONFIGURE
GO

EXEC sp_configure 'max degree of parallelism', 0;
RECONFIGURE
GO

EXEC sp_configure 'cost threshold for parallelism', 5;
RECONFIGURE
GO

*/

/*
	Examining the plan
	Check exchange operators and 
	row counts by thread
	ThreadStat info
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];


/*
	undocumented TF...only use for testing
*/
SELECT [h].[SalesOrderID], [d].[SalesOrderDetailID], [h].[OrderDate], [h].[CustomerID], [h].[SubTotal]
FROM [Sales].[SalesOrderHeader] [h]
JOIN [Sales].[SalesOrderDetail] [d] on [h].SalesOrderID = [d].SalesOrderID
WHERE [h].CustomerID > 700
OPTION (RECOMPILE, QUERYTRACEON 8649);



/*
	create a function to calculate sales price
*/
CREATE FUNCTION Sales.SalePrice(@UnitPrice MONEY, @Discount DECIMAL(4,2))
	RETURNS MONEY
	AS
	BEGIN
	
		DECLARE @MoneyOff MONEY = @UnitPrice * @Discount;

		RETURN (@UnitPrice - @MoneyOff)

	END
GO


/*
	same query, but function included in output
	what changes in the plan?
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], Sales.SalePrice([d].[UnitPrice],0.20) [SalePrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color]
OPTION (RECOMPILE, QUERYTRACEON 8649);



/*
	specify MAXDOP to restrict parallelism
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color]
OPTION (MAXDOP 1);


SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color]
OPTION (MAXDOP 2);