/*================================================================
  File:     07_Lookups.sql

  SQL Server Versions: 2008, 2008R2, 2012, 2014, 2016
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/
USE [AdventureWorks2016];
GO

/*
	create a copy of SalesOrderHeader and one nonclustered index
*/
SELECT *
INTO [Sales].[Copy_SalesOrderHeader]
FROM [Sales].[SalesOrderHeader];

CREATE NONCLUSTERED INDEX [NCI_Copy_SalesOrderHeader_SalesPersonID] 
	ON [Sales].[Copy_SalesOrderHeader] ([SalesPersonID]);


/*
	include actual plan
	Index Seek + RID Lookup (Heap)
*/
SELECT [SalesPersonID], [OrderDate], [SubTotal] 
FROM [Sales].[Copy_SalesOrderHeader]
WHERE [SalesPersonID] = 283;


/*
	Index Seek + Key Lookup (CI)
	What's the I/O?
*/
SELECT [SalesPersonID], [OrderDate], [SubTotal] 
FROM [Sales].[SalesOrderHeader]
WHERE [SalesPersonID] = 283;


/*
	Change the NCI to cover the query
	(Not a solution ALL the time!)
*/
CREATE NONCLUSTERED INDEX [IX_SalesOrderHeader_SalesPersonID] 
ON [Sales].[SalesOrderHeader]
(
	[SalesPersonID] ASC
)
INCLUDE ([OrderDate], [SubTotal])
WITH (DROP_EXISTING = ON)
ON [PRIMARY];
GO


/*
	re-run the query with the covering index
	what's the I/O?
*/
SELECT [SalesPersonID], [OrderDate], [SubTotal] 
FROM [Sales].[SalesOrderHeader]
WHERE [SalesPersonID] = 283;


/*
	What if we forced access just to the CI?
	What's the I/O?
*/
SELECT [SalesPersonID], [OrderDate], [SubTotal] 
FROM [Sales].[SalesOrderHeader]
WITH (INDEX(PK_SalesOrderHeader_SalesOrderID))
WHERE [SalesPersonID] = 283;


/*
	Change NCI back
*/
CREATE NONCLUSTERED INDEX [IX_SalesOrderHeader_SalesPersonID] 
ON [Sales].[SalesOrderHeader]
(
	[SalesPersonID] ASC
)
WITH (DROP_EXISTING = ON)
ON [PRIMARY];
GO


/*
	clean up
*/
DROP TABLE [Sales].[Copy_SalesOrderHeader];

