/*================================================================
  File:     08_Sorts.sql

  SQL Server Versions: 2008, 2008R2, 2012, 2014, 2016
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/
USE [AdventureWorks2016];
GO


/*
	SELECT query with a TOP 
*/
SELECT TOP 10 [CustomerID], [SubTotal]
FROM [Sales].[SalesOrderHeader];

/*
	SELECT query with a TOP, and an ORDER BY...
	because people usually want to see the data in some order, right?
*/
SELECT TOP 10 [CustomerID], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [SalesOrderID] DESC;

/*
	SELECT query with a TOP 
*/
SELECT TOP 10 [CustomerID], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [SubTotal] DESC;


/*
	variations in SORT operators (for reference)
	SORT vs. top N SORT vs. distinct SORT
	http://sqlperformance.com/2015/04/sql-plan/internals-of-the-seven-sql-server-sorts-part-1
	http://sqlperformance.com/2015/05/sql-plan/internals-of-the-seven-sql-server-sorts-part-2
*/
SELECT [CustomerID], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [SubTotal] DESC;

SELECT TOP 10 [CustomerID], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [SubTotal] DESC;

SELECT DISTINCT [CustomerID], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [SubTotal] DESC;


/*
	SELECT query with no ORDER BY 
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader];


/*
	SELECT query with an ORDER BY 
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader] 
ORDER BY [CustomerID];


/*
	What if I force the index?
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader] WITH (INDEX (IX_SalesOrderHeader_CustomerID))
ORDER BY [CustomerID];


/*
	run together to look at statistics
*/
SET STATISTICS IO ON;

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader] 
ORDER BY [CustomerID];

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader] WITH (INDEX (IX_SalesOrderHeader_CustomerID))
ORDER BY [CustomerID];

SET STATISTICS IO OFF;


/*
	Add a covering index to support the query
*/
CREATE NONCLUSTERED INDEX [IX_SalesOrderHeader_CustomerID_OrderDate_SubTotal] ON [Sales].[SalesOrderHeader](
	[CustomerID] ASC
	) 
	INCLUDE (
	[OrderDate], [SubTotal]
	)
	WITH (
	PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, 
	ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON
	) ON [PRIMARY];
GO


/*
	SELECT query with an ORDER BY, now with an index to support
*/
SET STATISTICS IO ON
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [CustomerID];
SET STATISTICS IO OFF

/*
	for tuning's sake, what's the IO improvement?
*/
SET STATISTICS IO ON;

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader] WITH (INDEX (PK_SalesOrderHeader_SalesOrderID))
ORDER BY [CustomerID];

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader] WITH (INDEX (IX_SalesOrderHeader_CustomerID))
ORDER BY [CustomerID];

SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [CustomerID];

SET STATISTICS IO OFF;


/*
	SELECT query with an ORDER BY, change the order to DESC
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [CustomerID] DESC;


/*
	Check the columns in the index
*/
EXEC sp_SQLskills_SQL2012_helpindex 'Sales.SalesOrderHeader';


/*
	What if we order on BOTH columns descending?
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [CustomerID] DESC, [SalesOrderID] DESC;


/*
	What if we have different sort orders?
*/
SELECT [CustomerID], [SalesOrderID], [OrderDate], [SubTotal]
FROM [Sales].[SalesOrderHeader]
ORDER BY [CustomerID] DESC, [SalesOrderID] ASC;


/*
	Remove our index
*/
DROP INDEX [IX_SalesOrderHeader_CustomerID_OrderDate_SubTotal] ON [Sales].[SalesOrderHeader]


