/*================================================================
  File:     09_Predicates.sql

  SQL Server Versions: 2008, 2008R2, 2012, 2014, 2016
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2017, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/
USE [AdventureWorks2016];
GO

/*
	Where is the filter?
	What kind of predicate?
*/
SELECT [ProductID], [Name], [ReorderPoint]
FROM [Production].[Product]
WHERE [ReorderPoint] > 250;


/*
	Now let's add an index on reorderpoint
*/
CREATE NONCLUSTERED
INDEX [NCI_Product_ReorderPoint] ON [Production].[Product] (
	[ReorderPoint]
	)
INCLUDE ([Name]);
GO

/*
	review of what's in the index
*/
EXEC sp_SQLskills_helpindex 'Production.Product';


/*
	Where is the filter?
	What kind of predicate?
*/
SELECT ProductID, Name, ReorderPoint
FROM [Production].[Product]
WHERE [ReorderPoint] > 250;


/*
	Look at a range
	Where is the filter?
	What kind of predicate?
*/
SELECT ProductID, Name, ReorderPoint
FROM [Production].[Product]
WHERE [ReorderPoint] BETWEEN 250 and 500;


/*
	Index Seek + Seek Predicate for IN?
*/
SELECT [ProductID], [Name], [ReorderPoint]
FROM [Production].[Product]
WHERE [ReorderPoint] IN (375, 600, 750)



/*
	Index Seek + Seek Predicate + Filter
	Does it push down the filter(s)?
	With the group by and aggregation, cannot filter until after
	If you see additional filters that don't expect, explore futher!

	Open in PE and show expressions
*/
SELECT [ReorderPoint], COUNT_BIG(*)
FROM [Production].[Product]
WHERE [ReorderPoint] IN (375, 600, 750)
GROUP BY [ReorderPoint]
HAVING COUNT_BIG(*) > 100


/*
	seek predicate
*/
SELECT 
	[e].[NationalIDNumber],
	[p].[Title],
	[p].[FirstName],
	[p].[MiddleName],
	[p].[LastName],
	[p].[Suffix],
	[e].[HireDate]
FROM [HumanResources].[Employee] [e]
JOIN [Person].[Person] [p] ON [p].[BusinessEntityID] = [e].[BusinessEntityID];

/*
	seek predicate and a predicate
	*note number of rows read!
*/
SELECT 
	[e].[NationalIDNumber],
	[p].[Title],
	[p].[FirstName],
	[p].[MiddleName],
	[p].[LastName],
	[p].[Suffix],
	[e].[HireDate]
FROM [HumanResources].[Employee] [e]
JOIN [Person].[Person] [p] ON [p].[BusinessEntityID] = [e].[BusinessEntityID]
WHERE [p].[LastName] = 'Williams';


/*
	residuals can appear in lookups...  
*/
SELECT soh.CustomerID, sod.ProductID
FROM Sales.SalesOrderDetail sod
INNER JOIN  Sales.SalesOrderHeader soh on sod.SalesOrderID = soh.SalesOrderID
WHERE soh.customerid = 29712;

SELECT soh.CustomerID, sod.ProductID
FROM Sales.SalesOrderDetail sod
INNER JOIN  Sales.SalesOrderHeader soh on sod.SalesOrderID = soh.SalesOrderID
WHERE soh.customerid = 29712
AND ShipMethodID = 5;


/*
	USE TF 9130 to push this out to a filter to see it better
	(don't use in production code)
*/
SELECT soh.CustomerID, sod.ProductID
FROM Sales.SalesOrderDetail sod
INNER JOIN  Sales.SalesOrderHeader soh on sod.SalesOrderID = soh.SalesOrderID
WHERE soh.customerid = 29712
AND ShipMethodID = 5
OPTION (QUERYTRACEON 9130)


/*
	and just in case, you can also find them in hash matches...
*/
SELECT [s].[Name], COUNT_BIG(*)
FROM [Production].[ProductSubcategory] s
INNER JOIN [Production].[Product] p
ON [s].[ProductSubcategoryID] = [p].[ProductSubcategoryID]
WHERE RIGHT([p].[Name], LEN([s].[Name])) = 'Locks'
GROUP BY s.Name; 

/*
	clean up
*/
DROP INDEX [Production].[Product].[NCI_Product_ReorderPoint];

